{
  GridView
  Copyright (C) 1996-2002 by Berg
  All rights reserved.

  $id:ColumnEditor.pas 01/04/2003 22:46:35 bn
}

{$R ButtonImagesRes.res}

unit ColumnEditor;

interface

uses
	DesignWindows, DesignIntf, Windows, Classes, Controls, ComCtrls, ImgList,
  SysUtils, Graphics, CustomGridView, Forms, Columns, CustomGridViewControl,
  ToolWin, ScrollView, CommCtrl, Dialogs, ThemesSupport, GridView, Menus;

type
  TColumnForm = class(TDesignWindow)
    ToolsImages: TImageList;
    MarksImages: TImageList;
    DesignerGrid: TGridView;
    TipImages: TImageList;
    PopupMenu1: TPopupMenu;
    ShowGrid1: TMenuItem;
    CoolBar1: TCoolBar;
    Tools: TToolBar;
    btnDelete: TToolButton;
    btnClear: TToolButton;
    ToolButton8: TToolButton;
    btnUp: TToolButton;
    btnDown: TToolButton;
    Toolbar: TToolBar;
    ToolbarImages: TImageList;
    procedure FormShow(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure DesignerGridCustomDrawCell(Sender: TObject; ACol,
      ARow: Integer; CellRect: TRect; CellState: TCellState);
    procedure FormDestroy(Sender: TObject);
    procedure DesignerGridSelectCell(Sender: TObject; ACol, ARow: Integer);
    procedure FormActivate(Sender: TObject);
    procedure DesignerGridKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DesignerGridDblClick(Sender: TObject);
    procedure ShowGrid1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FOwner: TPersistent;
    FDesigner: IDesigner;
    FEditingColumn: TCustomColumn;
    procedure SelectColumns;
    procedure SelectLastColumn;
    procedure SetOwner(const Value: TPersistent);
    procedure SetEditingColumn(const Value: TCustomColumn);
  protected
    function GetColumnImageIndex(ColumnClass: TColumnClass): Integer; virtual;
    procedure AddButton(ColumnCaption, ResString: string);
    procedure AddColumn(ColumnClass: TColumnClass); virtual;
    procedure ButtonClick(ColumnCaption: string); virtual;
    procedure CreateButtons; virtual;
    procedure DoButtonClick(Sender: TObject);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RefreshList;
  public
  	procedure UpdateColumnInEditor(Column: TCustomColumn);
    property Designer: IDesigner read FDesigner write FDesigner;
    property Owner: TPersistent read FOwner write SetOwner;
    property EditingColumn: TCustomColumn read FEditingColumn write SetEditingColumn;
  end;

implementation

uses
  ColumnClasses, Types;

{$R *.dfm}

{ TColumnForm }

procedure TColumnForm.FormCreate(Sender: TObject);
begin
  CreateButtons;
  btnUp.Enabled := False;
  btnDown.Enabled := False;
end;

procedure TColumnForm.AddButton(ColumnCaption, ResString: string);
var
  AButton: TToolButton;
  ABitmap: TBitmap;
begin
  AButton := TToolButton.Create(Self);
  AButton.Hint := ColumnCaption;
  AButton.ShowHint := True;
  AButton.ImageIndex := ToolbarImages.Count;
  AButton.OnClick := DoButtonClick;
  Toolbar.InsertControl(AButton);
  ABitmap := TBitmap.Create;
  ABitmap.LoadFromResourceName(HInstance, ResString);
  ToolbarImages.AddMasked(ABitmap, ABitmap.Canvas.Pixels[0, ABitmap.Height - 1]);
end;

procedure TColumnForm.AddColumn(ColumnClass: TColumnClass);
var
	AColumn: TCustomColumn;
begin
  AColumn := ColumnClass.Create(nil);
	try
  	with Owner as TCustomGridView do
    begin
      AColumn.Color := Color;
      AColumn.Font := Font;
      AColumn.Cursor := Cursor;
      AColumn.Position := Columns.Count;
		  Columns.AddColumn(AColumn);
    end;
  except
    AColumn.Free;
    raise;
  end;
  RefreshList;
  SelectLastColumn;
  btnUp.Enabled := DesignerGrid.RowCount > 1;
  Designer.Modified;
end;

function TColumnForm.GetColumnImageIndex(
  ColumnClass: TColumnClass): Integer;
begin
  Result := -1;
  if ColumnClass = TTextualColumn then Result := 10;
  if ColumnClass = TNumericColumn then Result := 9;
  if ColumnClass = TImageColumn then Result := 8;
  if ColumnClass = TCheckBoxColumn then Result := 7;
  if ColumnClass = TProgressColumn then Result := 6;
  if ColumnClass = TDateColumn then Result := 5;
  if ColumnClass = TComboBoxColumn then Result := 4;
  if ColumnClass = TIncrementColumn then Result := 3;
  if ColumnClass = TRateColumn then Result := 2;
  if ColumnClass = THtmlColumn then Result := 1;
  if ColumnClass = TGraphicColumn then Result := 0;
end;

procedure TColumnForm.ButtonClick(ColumnCaption: string);
begin
  if ColumnCaption = 'Textual Column' then AddColumn(TTextualColumn);
  if ColumnCaption = 'Numeric Column' then AddColumn(TNumericColumn);
  if ColumnCaption = 'Image Column' then AddColumn(TImageColumn);
  if ColumnCaption = 'CheckBox Column' then AddColumn(TCheckBoxColumn);
  if ColumnCaption = 'Progress Column' then AddColumn(TProgressColumn);
  if ColumnCaption = 'Date Column' then AddColumn(TDateColumn);
  if ColumnCaption = 'ComboBox Column' then AddColumn(TComboBoxColumn);
  if ColumnCaption = 'Increment Column' then AddColumn(TIncrementColumn);
  if ColumnCaption = 'Rate Column' then AddColumn(TRateColumn);
  if ColumnCaption = 'Html Column' then AddColumn(THtmlColumn);
  if ColumnCaption = 'Graphic Column' then AddColumn(TGraphicColumn);
end;

procedure TColumnForm.CreateButtons;
begin
  { Buttons need to be added in reverse order }
  AddButton('Graphic Column', 'TGRAPHICCOLUMN');
  AddButton('Html Column', 'THTMLCOLUMN');
  AddButton('Rate Column', 'TRATECOLUMN');
  AddButton('Increment Column', 'TINCREMENTCOLUMN');
  AddButton('ComboBox Column', 'TCOMBOBOXCOLUMN');
  AddButton('Date Column', 'TDATECOLUMN');
  AddButton('Progress Column', 'TPROGRESSCOLUMN');
  AddButton('CheckBox Column', 'TCHECKBOXCOLUMN');
  AddButton('Image Column', 'TIMAGECOLUMN');
  AddButton('Numeric Column', 'TNUMERICCOLUMN');
  AddButton('Textual Column', 'TTEXTUALCOLUMN');
end;

procedure TColumnForm.DoButtonClick(Sender: TObject);
begin
  with Sender as TToolButton do ButtonClick(Hint);
end;

procedure TColumnForm.RefreshList;
var
  i: Integer;
  Grid: TCustomGridView;
begin
  Grid := TCustomGridView(FOwner);
  DesignerGrid.ClearRows;
  for i := 0 to Grid.Columns.Count - 1 do
  begin
    DesignerGrid.AddRow;
    DesignerGrid.Cell[0, i].AsInteger := GetColumnImageIndex(TColumnClass(Grid.Columns[i].ClassType));

    if coFixedSize in Grid.Columns[i].Options
      then DesignerGrid.Cells[1, i] := '-1' else
    begin
      if coAutoSize in Grid.Columns[i].Options
        then DesignerGrid.Cells[1, i] := '1'
          else DesignerGrid.Cells[1, i] := '0';
    end;

    DesignerGrid.Cells[2, i] := '';

    DesignerGrid.Cell[3, i].AsInteger := -1;
    if (Grid.Columns[i] is TImageColumn) and (TImageColumn(Grid.Columns[i]).Images = nil) then
    begin
      DesignerGrid.Cell[3, i].AsInteger := 0;
      DesignerGrid.Cell[3, i].Hint := 'Images property is empty.';
    end;

    if (Grid.Columns[i] is TRateColumn) and (TRateColumn(Grid.Columns[i]).Glyph.Empty) then
    begin
      DesignerGrid.Cell[3, i].AsInteger := 0;
      DesignerGrid.Cell[3, i].Hint := 'Glyph property is empty.';
    end;

    DesignerGrid.Cells[4, i] := IntToStr(i) + ': ' + Grid.Columns[i].Header.Caption;
    if Grid.Columns[i].Name <> '' then DesignerGrid.Cells[4, i] := DesignerGrid.Cells[4, i] + ' (' + Grid.Columns[i].Name + ')';
    DesignerGrid.Cells[5, i] := Grid.Columns[i].ClassName;
  end;
  if DesignerGrid.RowCount > 0 then DesignerGrid.SelectedRow := DesignerGrid.RowCount - 1;
end;

procedure TColumnForm.SelectLastColumn;
begin
  with FOwner as TCustomGridView do Designer.SelectComponent(Columns[Columns.Count - 1]);
end;

procedure TColumnForm.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  { we need to free (and close) this window
    when FOwner (component) has been deleted }
  if (Operation = opRemove) and (AComponent = FOwner) then Free;
end;

procedure TColumnForm.SetEditingColumn(const Value: TCustomColumn);
begin
  FEditingColumn := Value;
end;

procedure TColumnForm.SetOwner(const Value: TPersistent);
begin
  FOwner := Value;
end;

procedure TColumnForm.FormShow(Sender: TObject);
begin
  { note: next line notify window (Self)
          that FOwner will be destroyed }
  TCustomGridView(FOwner).FreeNotification(Self);
  RefreshList;
  if DesignerGrid.RowCount > 0 then DesignerGrid.SelectedRow := 0;
  Caption := 'Column Editor - ' + TCustomGridView(FOwner).Name;
end;

procedure TColumnForm.btnDeleteClick(Sender: TObject);
var
  i: Integer;
begin
  Designer.Modified;
  if DesignerGrid.RowCount = 0 then Exit;
  { Note: Can't delete column because this control is
    showed in Object Inspector ! }
  Designer.NoSelection;

  i := 0;
  while i < DesignerGrid.RowCount do { delete all selected columns }
  begin
    if DesignerGrid.Selected[i] then
    begin
      DesignerGrid.DeleteRow(i);
		  TCustomGridView(FOwner).Columns.Delete(i);
    end else Inc(i);
  end;
  if (DesignerGrid.SelectedRow >= DesignerGrid.RowCount) and (DesignerGrid.RowCount > 0)
  	then DesignerGrid.SelectedRow := DesignerGrid.RowCount - 1;
end;

procedure TColumnForm.btnClearClick(Sender: TObject);
begin
  Designer.NoSelection;
  TCustomGridView(FOwner).Columns.Clear;
  DesignerGrid.ClearRows;
  Designer.Modified;
  btnUp.Enabled := False;
  btnDown.Enabled := False;
end;

procedure TColumnForm.btnUpClick(Sender: TObject);
begin
  DesignerGrid.MoveRow(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow - 1);
  with FOwner as TCustomGridView do
  begin
	  Columns.Move(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow - 1);
	  Columns.ChangePosition(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow - 1);
  end;
  DesignerGrid.SelectedRow := DesignerGrid.SelectedRow - 1;
  Designer.Modified;
end;

procedure TColumnForm.btnDownClick(Sender: TObject);
begin
  DesignerGrid.MoveRow(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow + 1);
  with FOwner as TCustomGridView do
  begin
  	Columns.Move(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow + 1);
  	Columns.ChangePosition(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow + 1);
	end;
  DesignerGrid.SelectedRow := DesignerGrid.SelectedRow + 1;
  Designer.Modified;
end;

procedure TColumnForm.DesignerGridCustomDrawCell(Sender: TObject; ACol,
  ARow: Integer; CellRect: TRect; CellState: TCellState);
var
  SmallRect: TRect;
begin
  if ACol = 2 then
  begin
    SmallRect := CellRect;
    InflateRect(SmallRect, -4, -2);
    DesignerGrid.Canvas.Pen.Color := clGrayText;
    DesignerGrid.Canvas.Brush.Color := TCustomGridView(FOwner).Columns[ARow].Color;
    DesignerGrid.Canvas.Rectangle(SmallRect);
  end;
  if ACol = 3 then
  begin
    if TCustomGridView(FOwner).Columns[ARow] is TProgressColumn then
    	with TCustomGridView(FOwner).Columns[ARow] as TProgressColumn do
	    begin
		    SmallRect := CellRect;
  	  	InflateRect(SmallRect, -4, -2);
		    DesignerGrid.Canvas.Pen.Color := BorderColor;
		    DesignerGrid.Canvas.Brush.Color := TCustomGridView(FOwner).Columns[ARow].Color;
		    DesignerGrid.Canvas.Rectangle(SmallRect);
		    DesignerGrid.Canvas.Brush.Color := ProgressColor;
  	  	InflateRect(SmallRect, -2, -2);
        SmallRect.Right := SmallRect.Right - 4;
        DesignerGrid.Canvas.FillRect(SmallRect);
	    end;
  end;
end;

procedure TColumnForm.FormDestroy(Sender: TObject);
begin
  TCustomGridView(FOwner).RemoveFreeNotification(Self);
end;

procedure TColumnForm.SelectColumns;
var
  CompList: IDesignerSelections;
  ColumnPersistent: TPersistent;
  i: Integer;
begin
  CompList := CreateSelectionList;
  for i := 0 to DesignerGrid.RowCount - 1 do
  	if DesignerGrid.Selected[i] then
		begin
      ColumnPersistent := TCustomGridView(FOwner).Columns[i];
			CompList.Add(ColumnPersistent);
	  end;
  if CompList.Count > 0 then
	begin
    EditingColumn := TCustomGridView(FOwner).Columns[DesignerGrid.SelectedRow];
	  Designer.SetSelections(CompList);
	end;
end;

procedure TColumnForm.DesignerGridSelectCell(Sender: TObject; ACol,
  ARow: Integer);
begin
  btnUp.Enabled := ARow > 0;
  btnDown.Enabled := ARow < DesignerGrid.RowCount - 1;
  if DesignerGrid.RowCount = 0 then Exit;
  SelectColumns;
end;

procedure TColumnForm.UpdateColumnInEditor(Column: TCustomColumn);
begin
  DesignerGrid.Cells[3, Column.Index] := IntToStr(Column.Index) + ': ' + Column.Header.Caption;
end;

procedure TColumnForm.FormActivate(Sender: TObject);
begin
  ActiveControl := DesignerGrid;
  SelectColumns;
end;

procedure TColumnForm.DesignerGridKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
	if Key = vk_delete then btnDeleteClick(btnDelete); 
end;

procedure TColumnForm.DesignerGridDblClick(Sender: TObject);
var
  CursorPoint, ACell: TPoint;
begin
  inherited;
  GetCursorPos(CursorPoint);
  CursorPoint := DesignerGrid.ScreenToClient(CursorPoint);
	ACell := DesignerGrid.GetCellAtPos(CursorPoint);
	if (ACell.X = 1) and (ACell.Y < DesignerGrid.RowCount) then
  begin
	  with FOwner as TCustomGridView do
	  	if coFixedSize in Columns[ACell.Y].Options then
	    begin
	    	DesignerGrid.Cells[ACell.X, ACell.Y] := '-1';
	      Exit;
	    end;
		if DesignerGrid.Cells[ACell.X, ACell.Y] = '10' then
    begin
    	DesignerGrid.Cells[ACell.X, ACell.Y] := '9';
  		with FOwner as TCustomGridView do Columns[ACell.Y].Options := Columns[ACell.Y].Options - [coAutoSize];
    end else
    begin
    	DesignerGrid.Cells[ACell.X, ACell.Y] := '10';
  		with FOwner as TCustomGridView do Columns[ACell.Y].Options := Columns[ACell.Y].Options + [coAutoSize];
    end;
  end;
end;

procedure TColumnForm.ShowGrid1Click(Sender: TObject);
begin
  if TMenuItem(Sender).Checked then DesignerGrid.Options := DesignerGrid.Options + [goGrid]
    else DesignerGrid.Options := DesignerGrid.Options - [goGrid];
end;

end.

